--[[=============================================================================
	File is: device_specific.lua  (Pakedge Camera Common)
	
    Copyright 2021 Wirepath Home Systems, LLC. All Rights Reserved.
===============================================================================]]


require "camera_proxy.camera_main"
require "url.url_main"
require "ip_server.ip_server_main"

require "pakedge_detection"
require "pakedge_email"
require "pakedge_firmware"
require "pakedge_camera"

require "device_specific_unique"

CAMERA_PROXY_BINDINGID = DEFAULT_PROXY_BINDINGID
NETWORK_BINDINGID = 6001

gSettingUp = false
gImageFlipped = false
gImageMirrored = false
gImageSize = "vga"
gNumberOfPresets = 0


-------------------------------------------------------------------------

function DeviceSpecificInitialize()
	LogTrace("DeviceSpecificInitialize")

	if(PersistData._UrlPersist == nil) then
		PersistData._UrlPersist = {}
		
		PersistData._UrlPersist.ADDRESS					= "localhost"
		PersistData._UrlPersist.HTTP_PORT               = C4:GetCapability("default_http_port")				or 80
		PersistData._UrlPersist.RTSP_PORT               = C4:GetCapability("default_rtsp_port")				or 554
		PersistData._UrlPersist.AUTHENTICATION_REQUIRED = toboolean(C4:GetCapability("default_authentication_required")) or false
		PersistData._UrlPersist.AUTHENTICATION_TYPE     = C4:GetCapability("default_authentication_type")	or "BASIC"
		PersistData._UrlPersist.USERNAME                = C4:GetCapability("default_username")				or "admin"
		PersistData._UrlPersist.PASSWORD                = C4:GetCapability("default_password")				or "pass"
	end
	
	gNumberOfPresets = tonumber(C4:GetCapability("number_presets")) or 0
	
	InitializeUrlCommunication(	PersistData._UrlPersist.USERNAME, 
						PersistData._UrlPersist.PASSWORD, 
						PersistData._UrlPersist.AUTHENTICATION_REQUIRED, 
						PersistData._UrlPersist.AUTHENTICATION_TYPE, 
						PersistData._UrlPersist.ADDRESS, 
						PersistData._UrlPersist.HTTP_PORT,
						PersistData._UrlPersist.RTSP_PORT
					)
					
	Pakedge_Detection.SetupDetections()
	
	SetupIPServer()
	C4:urlSetTimeout (20)

	gRetryTimer		= CreateTimer("Retry", 12, "SECONDS", OnRetryTimerExpired)
	gSettingsTimer	= CreateTimer("Settings", 5, "SECONDS", OnSettingsTimerExpired, false, "")
	gSyncTimer		= CreateTimer("Time Sync", 24, "HOURS", OnSyncTimerExpired, true)
	gRebootTimer	= CreateTimer("Reboot", 120, "SECONDS", OnRebootTimerExpired)
	gPingTimer		= CreateTimer("Ping", 240, "SECONDS", OnPingTimerExpired, true)
	
	-- Reconnect if we already have a connection for driver update
	local ip = C4:GetBindingAddress(NETWORK_BINDINGID)
	if (ip ~= nil and string.len(ip) > 0) then 
		OnNetworkBindingChanged(NETWORK_BINDINGID, true)
	else
		Camera:SetOnlineStatus(NETWORK_BINDINGID, false, true)
	end

	ReportPropertyDefaults()
	EmailControl:UpdateEmailProperties()
	FirmwareControl:Initialize()
	
end


function DeviceOnlineStatusChanged(IsOnline)
	LogTrace("DeviceOnlineStatusChanged  Online is: %s", tostring(IsOnline))
	if(IsOnline) then
		-- just came online
	else
		-- just went offline
	end
end


function OnDriverDestroyed()
	LogTrace("OnDriverDestroyed")

end

function OnNetworkBindingChanged (idBinding, bIsBound)
	LogTrace("OnNetworkBindingChanged %d  %s", idBinding, tostring(bIsBound))

	if (idBinding == NETWORK_BINDINGID) then
		if (bIsBound) then
			local NAddress = C4:GetBindingAddress (idBinding)
			UrlSetAddress(NAddress)
			local controllerip = C4:GetControllerNetworkAddress()
			CameraReport_AddressChanged(NAddress)

			local _, _, ip_range = string.find(NAddress, '(%d+.%d+.%d+).%d')
			local _, _, controller_range = string.find(controllerip, '(%d+.%d+.%d+).%d')
			if (ip_range ~= controller_range) then
				LogWarn('WARNING: Camera IP address not on same subnet as C4')
			else
				LogInfo('Attempting to connect to camera')
				Camera:SetOnlineStatus(idBinding, true, true)
				gPingTimer:StartTimer()
			end
		else
			Camera:SetOnlineStatus(idBinding, false, true)
			gPingTimer:KillTimer()
		end
	end
end


--[[=============================================================================
    OnBindingChanged(idBinding, class, bIsBound)
  
    Description:
    Function called by Director when a binding changes state(bound or unbound).
  
    Parameters:
    idBinding(int) - ID of the binding whose state has changed.
    class(string)  - Class of binding that has changed.
                     A single binding can have multiple classes(i.e. COMPONENT,
                     STEREO, RS_232, etc).
                     This indicates which has been bound or unbound.
    bIsBound(bool) - Whether the binding has been bound or unbound.
  
    Returns:
    None
===============================================================================]]
function OnBindingChanged(idBinding, class, bIsBound)
	LogTrace("OnConnectionChanged  Binding: %d  Class: %s  Bound: %s", tonumber(idBinding), tostring(class), tostring(bIsBound))
	
	if(bIsBound) then
		-- Do bind stuff
	else
		-- Do unbind stuff
	end
end


function SyncCameraTime()
	local nowtime = os.date("%X")
	local nowdate = os.date("%Y/%m/%d")
	LogTrace("Time Sync " .. nowtime .. " " .. nowdate)
	local url = BuildHTTPString("setclock?Time=" .. nowtime .. "&Date=" .. nowdate )
	UrlGet(url)
end


function ShowDriverInfo(InfoMessage)
	C4:UpdateProperty("Driver Information", InfoMessage)
end


function SyncEmailSettings()
	if(EmailControl:IsEnabled()) then
		EmailControl:Update()
	end
end


--=========================================================================
--=========================================================================
--  Properties

function ON_PROPERTY_CHANGED.CameraID(propertyValue)
	SetCameraID(tonumber(propertyValue))
	Camera:QuerySettings()
end

function ON_PROPERTY_CHANGED.MJPEGImageSize(propertyValue)
	LogTrace("ON_PROPERTY.MjpegImageSize: " .. propertyValue)
	gImageSize = propertyValue
end

function ON_PROPERTY_CHANGED.SyncwithC4Time(propertyValue)
	LogTrace("ON_PROPERTY.SyncTime: " .. propertyValue)
	if (propertyValue == "Yes") then
		gSyncTimer:StartTimer()
		SyncCameraTime()
	else
		gSyncTimer:KillTimer()
	end
end

function ON_PROPERTY_CHANGED.FirmwareImageURL(propertyValue)
	LogTrace("ON_PROPERTY.FirmwareImageURL: " .. propertyValue)
	FirmwareControl:SetFirmwareImageURL(propertyValue)
end

--=========================================================================
--=========================================================================
--  Actions

function LUA_ACTION.GetCameraResolutions()
	Camera:QueryCameraResolutions()
end


function LUA_ACTION.Flip ()
	if (gImageFlipped) then
		EX_CMD.UNFLIP_IMAGE()
	else
		EX_CMD.FLIP_IMAGE()
	end
end

function LUA_ACTION.Mirror ()
	if (gImageMirrored) then
		EX_CMD.UNMIRROR_IMAGE()
	else
		EX_CMD.MIRROR_IMAGE()
	end
end

function LUA_ACTION.Reboot ()
	EX_CMD.REBOOT_CAMERA()
end

function LUA_ACTION.Reinitialize ()
	gSettingUp = true
	Camera:ClearInfo()
	Camera:GetSettings()
	SyncCameraTime()
end

function LUA_ACTION.Report ()
	Camera:ClearInfo()
	Camera:GetSettings()
end

function LUA_ACTION.Snapshot ()
	EX_CMD.SEND_SNAP_SHOT ()
end

function LUA_ACTION.EnableMotion()
	EX_CMD.SEND_Enable_Motion ()
end

function LUA_ACTION.DisableMotion()
	EX_CMD.SEND_Disable_Motion ()
end

function LUA_ACTION.Time ()
	SyncCameraTime()
end

function LUA_ACTION.SyncEmail ()
	SyncEmailSettings()
end


function LUA_ACTION.UpdateFirmware()
	FirmwareControl:Update()
end

function LUA_ACTION.DefaultFirmwareURL()
	FirmwareControl:SetDefaultFirmwareURL()
end

function LUA_ACTION.BACK_LIGHT_COMPENSATION_ON()
	local url = ""
	url = BuildHTTPString("quality_apply?cmd=set&BLC=1&camid=" .. GetCameraID())
	UrlGet(url)
end	
function LUA_ACTION.BACK_LIGHT_COMPENSATION_OFF()
	local url = ""	
	url = BuildHTTPString("quality_apply?cmd=set&BLC=0&camid=" .. GetCameraID())
	UrlGet(url)
end


--=========================================================================
--=========================================================================
-- Commands

function EX_CMD.Reinitialize()
	gSettingUp = true
	Camera:ClearInfo()
	Camera:GetSettings()
	SyncCameraTime()
end		 

function EX_CMD.TIME_SYNC()
	SyncCameraTime()
end

function EX_CMD.AUTO_FOCUS()
	local url = BuildHTTPString("camera?autofocus=1&camid=" .. GetCameraID())
	UrlGet(url)
end

function EX_CMD.AUTO_IRIS()
	local url = BuildHTTPString("camera?autoiris=1&camid=" .. GetCameraID())
	UrlGet(url)
end

function EX_CMD.FLIP_IMAGE()
	local url = BuildHTTPString("quality_apply?cmd=set&FLIP=1&camid=" .. GetCameraID())
	UrlGet(url)
	gImageFlipped = true
end

function EX_CMD.UNFLIP_IMAGE()
	local url = BuildHTTPString("quality_apply?cmd=set&FLIP=0&camid=" .. GetCameraID())
	UrlGet(url)
	gImageFlipped = false
end

function EX_CMD.MIRROR_IMAGE()
	local url = BuildHTTPString("quality_apply?cmd=set&MIRROR=1&camid=" .. GetCameraID())
	UrlGet(url)
	gImageMirrored = true
end

function EX_CMD.UNMIRROR_IMAGE()
	local url = BuildHTTPString("quality_apply?cmd=set&MIRROR=0&camid=" .. GetCameraID())
	UrlGet(url)
	gImageMirrored = false
end

function EX_CMD.REBOOT_CAMERA()
	local url = BuildHTTPString('reboot')
	UrlGet(url)
	LogWarn('Camera rebooting - will query it in 120 seconds')
	Camera:ClearInfo()
	gRebootTimer:StartTimer()
end


function EX_CMD.INC_IMAGE_VALUE (tParams)
	local url = BuildHTTPString("camera?" .. tParams.Type .. "=1&camid=" .. GetCameraID())
	UrlGet(url)
	local url = BuildHTTPString("camera?stop=1&camid=" .. GetCameraID())
	UrlGet(url)
end

function EX_CMD.DEC_IMAGE_VALUE (tParams)
	local url = BuildHTTPString("camera?" .. tParams.Type .. "=-1&camid=" .. GetCameraID())
	UrlGet(url)
	local url = BuildHTTPString("camera?stop=1&camid=" .. GetCameraID())
	UrlGet(url)
end

function EX_CMD.DAY_NIGHT (tParams)
--	CameraCom_NightMode(tParams.ToD ~= 'Day')
	
	local IRParam = "auto"
	if(tParams.ToD == 'Day') then IRParam = "Disable" end
	if(tParams.ToD == 'Night') then IRParam = "Enable" end
	
	EX_CMD.ENABLE_IR({Value = IRParam})
end

function EX_CMD.BACK_LIGHT_COMPENSATION(tParams)
	local url = ""
	if (tParams.Value == 'On') then
		url = BuildHTTPString("quality_apply?cmd=set&BLC=1&camid=" .. GetCameraID())
	else
		url = BuildHTTPString("quality_apply?cmd=set&BLC=0&camid=" .. GetCameraID())
	end
	
	UrlGet(url)
end

function EX_CMD.WHITE_BALANCE (tParams)
	Camera:SetWhiteBalanceFromString(tParams.Type)
end

function EX_CMD.EXPOSURE (tParams)
	local url = BuildHTTPString("quality_apply?cmd=set&EV=" .. tParams['Value (1-16)'] .. "&camid=" .. GetCameraID())
	UrlGet(url)
end

function EX_CMD.NOISE_REDUCTION_3D (tParams)
	local url = BuildHTTPString("quality_apply?cmd=set&NoiseReduction=" .. tParams['Value (0-32)'] .. "&camid=" .. GetCameraID())
	UrlGet(url)
end

function EX_CMD.AUTO_GAIN_CONTROL (tParams)
	local url = ""
	if (tParams.Gain == '6dB') then
		url = BuildHTTPString("quality_apply?cmd=set&AGC=1&camid=" .. GetCameraID())
	elseif (tParams.Gain == '12dB') then
		url = BuildHTTPString("quality_apply?cmd=set&AGC=2&camid=" .. GetCameraID())
	elseif (tParams.Gain == '18dB') then
		url = BuildHTTPString("quality_apply?cmd=set&AGC=3&camid=" .. GetCameraID())
	elseif (tParams.Gain == '24dB') then
		url = BuildHTTPString("quality_apply?cmd=set&AGC=4&camid=" .. GetCameraID())
	elseif (tParams.Gain == '30dB') then
		url = BuildHTTPString("quality_apply?cmd=set&AGC=5&camid=" .. GetCameraID())
	elseif (tParams.Gain == '36dB') then
		url = BuildHTTPString("quality_apply?cmd=set&AGC=6&camid=" .. GetCameraID())
	end

	UrlGet(url)
end

function EX_CMD.ENABLE_IR (tParams)
	LogTrace("EX_CMD.ENABLE_IR")

	local url = ""
	if (tParams.Value == 'Enable') then
		url = BuildHTTPString("ir=on")
	elseif (tParams.Value == 'Disable') then
		url = BuildHTTPString("ir=off")
	else -- auto
		url = BuildHTTPString("ir=auto")
	end

	UrlGet(url)
end

function EX_CMD.SEND_SNAP_SHOT (tParams)
	EmailControl:SendSnapshot()
end

function EX_CMD.SEND_Enable_Motion (tParams)
	local url = BuildHTTPString('mdmotion?MD_FLAG=1')
	UrlGet(url)
end

function EX_CMD.SEND_Disable_Motion (tParams)
	local url = BuildHTTPString('mdmotion?MD_FLAG=0')
	UrlGet(url)
end

function EX_CMD.SEND_Enable_Alarm (tParams)
	local url = BuildHTTPString('mdmotion?ALARM_FLAG=1')
	UrlGet(url)
end

function EX_CMD.SEND_Disable_Alarm (tParams)
	local url = BuildHTTPString('mdmotion?ALARM_FLAG=0')
	UrlGet(url)
end

function EX_CMD.SEND_Enable_Audio (tParams)
	local url = BuildHTTPString('apply.cgi?action=ad_apply&next_page=audiodetect.asp&AD_FLAG=1')
	UrlGet(url)
end

function EX_CMD.SEND_Disable_Audio (tParams)
	local url = BuildHTTPString('apply.cgi?action=ad_apply&next_page=audiodetect.asp&AD_FLAG=0')
	UrlGet(url)
end

function EX_CMD.SET_EMAIL (tParams)
	LogTrace("EX_CMD.SET_EMAIL")
	EmailControl:SetToAddress(tParams.Email)
	UpdateProperty("Email To Address", tParams.Email)
end

function EX_CMD.SET_EMAIL_SUBJECT (tParams)
	LogTrace("EX_CMD.SET_EMAIL_SUBJECT")
	EmailControl:SetSubject(tParams.Subject)
end

function EX_CMD.SET_EMAIL_MESSAGE (tParams)
	LogTrace("EX_CMD.SET_EMAIL_MESSAGE")
	EmailControl:SetMessage(tParams.Message)
end


--=========================================================================
--=========================================================================
--  Timer Expirations

function OnRetryTimerExpired()
	LogTrace("OnRetryTimerExpired")
	Camera:GetSettings()
end


function OnSettingsTimerExpired()
	Camera:GetSettings ()
end

function OnSyncTimerExpired()
	SyncCameraTime()
	Camera:QueryCameraResolutions()
	SyncEmailSettings()
end

function OnRebootTimerExpired()
	LogWarn("Reboot Timer Expired -- Get Settings")
	Camera:GetSettings ()
end


function OnPingTimerExpired()
	LogTrace("OnPingTimerExpired")
	if(gRebootTimer:TimerStopped() and (not FirmwareControl:IsUpdating())) then	-- don't try pinging in the middle of a reboot or firmware update
		Camera:SendPing()
	end
end

ld_DriverSpecific = true		--- for development

