--[[=============================================================================
	File is: pakedge_camera.lua  (Pakedge Camera)
	
    Copyright 2021 Wirepath Home Systems, LLC. All Rights Reserved.
===============================================================================]]


Camera = {
	_IsOnline = false,
	_SettingsValid = false,

	_RetryCount = 0,
	_MAX_RETRIES = 15,
	
	_CameraDeviceName = "",
	
	_HasMotionSensor = false,
	_MotionSensitivity = "Standard",

	_SensitivityLevelMap = {		
		{ text = "Highest",			value = 1 },
		{ text = "Sensitivity 7",	value = 15 },
		{ text = "Sensitivity 6",	value = 30 },
		{ text = "Sensitivity 5",	value = 43 },
		{ text = "Standard",		value = 57 },
		{ text = "Sensitivity 3",	value = 71 },
		{ text = "Sensitivity 2",	value = 85 },
		{ text = "Low",				value = 99 },
	},
	
	_WhiteBalance = 0,			-- default to 'Auto'
	_WhiteBalanceMap = {
		[0] = "Auto",
		[1] = "Tungsten",
		[2] = "Fixed Indoor",
		[3] = "Fixed Fluorescents 1",
		[4] = "Fixed Fluorescents 2",
		[5] = "Fixed Outdoor 1",
		[6] = "Fixed Outdoor 2",
	}

}


gH264TEXT = {}
gJPEGTEXT = {}


-- LILIN feature variables
SYS_ALARM_IN		= 0x01		-- Alarm In
SYS_ALARM_OUT		= 0x02		-- Alarm Out
SYS_IR_REV			= 0x04		-- IR Cut
SYS_IR_REV_EXT		= 0x08		-- IR Cut external input
SYS_TV_OUT			= 0x10		-- analogue output
SYS_EXT_PTZ			= 0x20		-- IP PTZ, Type = 1
SYS_SD_CARD			= 0x40		-- SD card
SYS_AUTO_FOCUS		= 0x80		-- Auto focus camera
SYS_AUD_IN			= 0x100		-- Audio in model
SYS_AUD_MIC_IN		= 0x200		-- MIC input
SYS_AUD_LINE_IN		= 0x400		-- Line in
SYS_AUD_OUT			= 0x800		-- Line out/speaker out
SYS_ALARM_MOTION	= 0x1000	-- Motion detection
SYS_ALARM_FACE		= 0x2000	-- Face detection
SYS_ALARM_AUDIO		= 0x4000	-- Audio detection
SYS_ALARM_TAMPER	= 0x8000	-- Tampering detection
SYS_ALARM_NET		= 0x10000	-- Network lose detection
SYS_ROI_RPTZ		= 0x20000	-- ROI, ePTZ, Type = 2
SYS_PRI_MASK		= 0x40000	--//Mask feature
SYS_SENSOR_2710		= 0x80000	-- Reserve
SYS_VARIFOCAL		= 0x100000	-- Auto focus
SYS_FISHEYE			= 0x200000	--// Fisheye, Type = 3
SYS_IRLED			= 0x400000	--//Reserve
SYS_ALARM_WIRE		= 0x800000	-- Cross wire detection
SYS_COLOR_LINE		= 0x1000000	-- Color and wire detection
SYS_PEOPLECOUNT		= 0x2000000	-- People counting
SYS_TRACKING		= 0x4000000	-- Tracking speed dome



--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Camera functions
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=


function Camera:SetOnlineStatus(idBinding, NowOnline, ForceCheck)

	if((NowOnline ~= self._IsOnline) or ForceCheck) then
		self._IsOnline = NowOnline
		
		if(self._IsOnline) then
			self:GetSettings()
			C4:SetBindingStatus(idBinding, "online")
			ShowDriverInfo("Online")
			SyncEmailSettings()
			SyncCameraTime()
		else
			self:ClearInfo()
			C4:SetBindingStatus(idBinding, "offline")
			ShowDriverInfo("Offline")
		end
	end
end


function Camera:IsOnline()
	return self._IsOnline
end


function Camera:SettingsAreValid()
	return self._SettingsValid
end


function Camera:SetSettingsValidFlag(AreValid)
	self._SettingsValid = AreValid
end


function Camera:ClearInfo()
	self:SetSettingsValidFlag(false)
	self:SetCameraDeviceName("")
	FirmwareControl:SetModelName("")
	FirmwareControl:SetVersion(' ')
	C4:UpdateProperty ('Detections', ' ')
	C4:UpdateProperty ('PTZ', ' ')
	self:SetMotionSensorFlag(false)
end


function Camera:RetryCountNotExceeded()
	return (self._RetryCount < self._MAX_RETRIES)
end

		
function Camera:BumpRetryCount()
	self._RetryCount = self._RetryCount + 1
	LogTrace("Camera:BumpRetryCount  Now: %d", self._RetryCount)
end


function Camera:GetSettings()
	LogTrace("Camera:GetSettings")

	-- Get all camera settings as required
	self:ClearInfo()

	UrlGet(BuildHTTPString('server'), Resp_GetSettings)

	self:QueryCameraResolutions()
end


function Camera:GetSettingsSuccess(data)
	self:SetSettingsValidFlag(true)
	
	if(FirmwareControl:IsUpdating()) then
		FirmwareControl:ResumedCommunication()
	else
		ShowDriverInfo("Communication Established")
	end
	
	local WasntDiscovered = (self:GetCameraDeviceName() == "")
	self._RetryCount = 0
	
	local CameraModelName = string.match(data, "Model Name=([%C]+)%c")
	local DeviceName = string.match(data, "device name=([%C]+)%c")
	local version = string.match(data, "Software Version=([%C]+)%c")
	version = version .. "." .. string.match(data, "SVN_VERSION=([%C]+)%c")
	local dhcp = string.match(data, "DhcpEnable=(%d+)")
	local SystemFeaturesBits = string.match(data, "SysFeature=(%d+)")

	self:SetCameraDeviceName(DeviceName)
	FirmwareControl:SetModelName(CameraModelName)
	FirmwareControl:SetVersion(version)

	if (dhcp == '1') then
		LogInfo("Camera set to DHCP mode, this is OK if using SDDP")
	else
		LogInfo("Camera set to Static IP")
	end

	local DetectionsList = ""
	local PtzList = ""

	if (gSettingUp) then
		Pakedge_Detection.ClearAllDetections()
	end

	LogDebug("SystemFeaturesBits: %08X", SystemFeaturesBits)
	
	local function ConjureUpNewDetection(DetectionName, FeatureID)
		if(not Pakedge_Detection.DetectionExists(DetectionName)) then
			Pakedge_Detection.AddNewDetection(DetectionName)

		end

		if(string.find(DetectionsList, DetectionName) == nil) then
			DetectionsList = DetectionsList .. DetectionName .. " "
		end
		
		self:StartServer(FeatureID, DetectionName, '')
	end

	local CanDoMotionSensing = false
	
	if (bit.band(SystemFeaturesBits, SYS_ALARM_MOTION) == SYS_ALARM_MOTION) then
		ConjureUpNewDetection("Motion", SYS_ALARM_MOTION)
		CanDoMotionSensing = true
	end

	if (bit.band(SystemFeaturesBits, SYS_ALARM_FACE) == SYS_ALARM_FACE) then
		ConjureUpNewDetection("Face", SYS_ALARM_FACE)
	end

	if (bit.band(SystemFeaturesBits, SYS_ALARM_AUDIO) == SYS_ALARM_AUDIO) then
		ConjureUpNewDetection("Audio", SYS_ALARM_AUDIO)
	end

	if (bit.band(SystemFeaturesBits, SYS_ALARM_TAMPER) == SYS_ALARM_TAMPER) then
		ConjureUpNewDetection("Tamper", SYS_ALARM_TAMPER)
	end

	if (bit.band(SystemFeaturesBits, SYS_ALARM_IN) == SYS_ALARM_IN) then
		ConjureUpNewDetection("Alarm", SYS_ALARM_IN)
	end

	if (bit.band(SystemFeaturesBits, SYS_ALARM_WIRE) == SYS_ALARM_WIRE) then
		--ConjureUpNewDetection("tripwire", SYS_ALARM_WIRE)
		-- The Trip Wire events show up as motion events
		ConjureUpNewDetection("Motion", SYS_ALARM_MOTION)
		CanDoMotionSensing = true
	end

	--  We have no good way to handle these notifications
	--if (bit.band(SystemFeaturesBits, SYS_COLOR_LINE) == SYS_COLOR_LINE) then
	--	ConjureUpNewDetection("color/line", SYS_COLOR_LINE)
	--end

	--  We have no good way to handle these notifications
	--if (bit.band(SystemFeaturesBits, SYS_PEOPLECOUNT) == SYS_PEOPLECOUNT) then
	--	ConjureUpNewDetection("object count", SYS_PEOPLECOUNT)
	--end

	if (bit.band(SystemFeaturesBits, SYS_VARIFOCAL) == SYS_VARIFOCAL) then
		LogInfo("Has Variable Focus")
	else
		LogInfo("This is not a variable focus camera - focus functions will not work")
	end

	if(gThisIsAPTZDriver) then
		if (bit.band(SystemFeaturesBits, SYS_EXT_PTZ) == SYS_EXT_PTZ) then
			PtzList = PtzList .. "IP "
		end

		if (bit.band(SystemFeaturesBits, SYS_AUTO_FOCUS) == SYS_AUTO_FOCUS) then
			PtzList = PtzList .. "Auto focus "
			SetAutoFocusFlag(true)
		else
			SetAutoFocusFlag(false)
		end	

		if (bit.band(SystemFeaturesBits, SYS_ROI_RPTZ) == SYS_ROI_RPTZ) then
			SetEPTZFlag(true)
			PtzList = PtzList .. "ePTZ "
		else
			SetEPTZFlag(false)
		end
	else
		SetAutoFocusFlag(false)
		SetEPTZFlag(false)
	end

	if (bit.band(SystemFeaturesBits, SYS_IR_REV) == SYS_IR_REV) then
		LogInfo("Has IR LEDs")
	else
		LogInfo("Does not have IR LEDs - IR LED commands will not work")
	end

	if (bit.band(SystemFeaturesBits, SYS_ALARM_OUT) == SYS_ALARM_OUT) then
		LogInfo("Has Alarm Out")
	end

	if (bit.band(SystemFeaturesBits, SYS_FISHEYE) == SYS_FISHEYE) then
		LogInfo("Has Fisheye Lens")
	end

	if (bit.band(SystemFeaturesBits, SYS_TRACKING) == SYS_TRACKING) then
		LogInfo("Has tracking Dome")
	end

	self:SetMotionSensorFlag(CanDoMotionSensing)

	C4:UpdateProperty ('Detections', DetectionsList)
	LogInfo("Camera supports the following detections: %s", DetectionsList)

	if(gThisIsAPTZDriver) then
		if (PtzList == "") then
			C4:UpdateProperty ('PTZ', "NO PTZ - WRONG DRIVER!")
			LogWarn('This is the WRONG DRIVER for this camera as it does not have PTZ')
		else
			C4:UpdateProperty ('PTZ', PtzList)
			LogInfo('Camera has ' .. PtzList .. ' PTZ')
			if (HasEPTZ()) then
				LogInfo('NOTE: As camera has ePTZ facilities the secondary D1 stream will be used.  Pan and Tilt functions first require you to zoom in on the image.')
				LogInfo('If you do not want the ePTZ functions then use the Static Camera driver instead and manually identify the camera instead of using SDDP.')
				SetCameraID(2)
			else
				SetCameraID(1)
			end
		end
	else
		SetCameraID(1)	
	end

	if(WasntDiscovered) then
		LogInfo("Camera: %s discovered", Camera:GetCameraDeviceName())
	end

	gSettingUp = false	 -- finished setting up now
end


function Camera:QuerySettings()
	if(self:IsOnline()) then
		gSettingsTimer:StartTimer()
	end
end


function Camera:GetSettingsFailure(responsecode)
	local ResponseString = GetURLResponseString(responsecode)
	LogWarn("GetSettingsFailure: %s", ResponseString)
	ShowDriverInfo(string.format("Communication Failure: %s", ResponseString))

	if(self:RetryCountNotExceeded()) then
		self:BumpRetryCount()
		gRetryTimer:StartTimer()
	else
		LogWarn("Communication Retry Count exceeded.  Not established")
	end
end


function Camera:QueryCameraResolutions()
	UrlGet(BuildHTTPString('geturl'), Resp_GetURLSettings)
end


function Camera:TriggerAlert (detection)
	if(Pakedge_Detection.DetectionExists(detection)) then
		Pakedge_Detection.StartDetection(detection)
	else
		LogWarn('No detection contact %s defined', detection)
	end
end

function Camera:StartServer (feature, name, addnum) -- Fix the issue of http post cgi  release 1.1.1
	local ip = C4:GetControllerNetworkAddress ()
	local url = BuildHTTPString('sethttppost?destip=' .. ip .. '&destport=' .. gServerPort .. '&url=%2F' .. name .. '&SysFeature=0x' .. addnum .. string.format("%X", feature) .. '&httppost_en=1&httppostjpeg_en=0&' .. name ..'_en=1&account=LILIN&password=control4&dwell=5')
	UrlGet(url)
end


function Camera:SendPing()
	local url = BuildHTTPString(string.format("quality?cmd=get&DayNight=0"))
	UrlGet(url, Resp_PingResponse)
end




function Camera:SetCameraDeviceName(DeviceName)
	self._CameraDeviceName = DeviceName
	
	C4:UpdateProperty ('Camera Device Name', self._CameraDeviceName)
	LogInfo('Camera Device Name: ))%s((', self._CameraDeviceName)
end

function Camera:GetCameraDeviceName()
	return self._CameraDeviceName
end



--------------

function Camera:QueryMotionSettings()
	local url = BuildHTTPString(string.format("motion?cmd=get"))
	UrlGet(url, Resp_MotionSettings)
end

function Camera:SetMotionSensorFlag(NewFlag)
	self._HasMotionSensor = NewFlag
	
	C4:SetPropertyAttribs("Motion Sensitivity",	self._HasMotionSensor and 0 or 1)		-- 0 -> Show   1 --> Hide
	
	if(self._HasMotionSensor) then
		self:QueryMotionSettings()
	end
end

function Camera:HasMotionSensor()
	return self._HasMotionSensor
end

function Camera:SetMotionSensitivity(NewSensitivity)

	local SensitivityIndex = 0
	for LIndex = 1, #self._SensitivityLevelMap do
		if(self._SensitivityLevelMap[LIndex].text == NewSensitivity) then
			SensitivityIndex = LIndex
			break
		end
	end

	self._MotionSensitivity = NewSensitivity
	
	local url = BuildHTTPString(string.format("motion?cmd=set&mdSen=%d", self._SensitivityLevelMap[SensitivityIndex].value))
	LogTrace("Camera:SetMotionSensitivity  ))%s((", url)
	UrlGet(url, Resp_MotionSettings)
end


function Camera:MotionSensitivityFromCamera(SenValue)
	local SensitivityIndex = 1

	for k, v in ipairs(self._SensitivityLevelMap) do
		if(SenValue >= v.value) then
			SensitivityIndex = k
		end
	end

	self._MotionSensitivity = self._SensitivityLevelMap[SensitivityIndex].text
	LogTrace("Camera:MotionSensitivityFromCamera Value is %d  Level is: %s", SenValue, self._MotionSensitivity)
	
	C4:UpdateProperty("Motion Sensitivity", self._MotionSensitivity)
end



function ON_PROPERTY_CHANGED.MotionSensitivity(propertyValue)
	LogTrace("ON_PROPERTY.MotionSensitivity: %s", propertyValue)
	Camera:SetMotionSensitivity(propertyValue)
end


-------------

function Camera:SetWhiteBalanceFromString(BalanceStr)
	LogTrace("Camera:SetWhiteBalanceFromString  %s", BalanceStr)
	local BIndex = 0
	
	for k,v in ipairs(self._WhiteBalanceMap) do
		if(BalanceStr == v) then
			BIndex = k
			break
		end
	end

	local url = BuildHTTPString(string.format("quality?cmd=set&AWB=%d&camid=%d", BIndex, GetCameraID()))
	UrlGet(url, Resp_GetQuality)
end


function ON_PROPERTY_CHANGED.WhiteBalance(propertyValue)
	LogTrace("ON_PROPERTY.WhiteBalance: %s", propertyValue)
	Camera:SetWhiteBalanceFromString(propertyValue)
end

function Camera:SaveWhiteBalance(BalanceValue)
	LogTrace("Camera:SaveWhiteBalance  %s", BalanceValue)
	self._WhiteBalance = BalanceValue
	C4:UpdateProperty("White Balance", self._WhiteBalanceMap[self._WhiteBalance])
end


----------------------Push Notifications-------------------------------
--Implementing at latter date.
function BuildHTTPStringAttachURL(InString)
	return TheUrlCom:BuildHTTPAttachURL(InString)
end

function UrlCom:BuildHTTPAttachURL(query_string)
	return string.format("http://%s:%s@%s:%s/%s", TheUrlCom:GetUserName(), TheUrlCom:GetPassword(), self:GetAddress(), self:GetHttpPort(), query_string)
end

function GetNotificationAttachmentURL()
    LogTrace("(GetNotificationAttachmentURL)")
    local SnapQueryString = string.format("snap%d%s", GetCameraID(), "vga")
    LogTrace("Url: %s", SnapQueryString)
    return BuildHTTPStringAttachURL(SnapQueryString)
end

--------------------------------------------------

function TryCom(Command)
	local url = BuildHTTPString(Command)
	LogInfo("TryCom: %s", url)
end

function Resp_TryComResponse(ResponseCode, Data)
	LogInfo("Resp_TryComResponse Response: %s  Data >>%s<<", tostring(ResponseCode), tostring(Data))
end


ld_pakedge_camera = true		-- for development
