--[[=============================================================================
	File is: pakedge_firmware.lua
	
    Copyright 2021 Wirepath Home Systems, LLC. All Rights Reserved.
===============================================================================]]



FIRMWARE_UPGRADE_STRING = "upgrade.cgi?type=upfs"


FirmwareControl = {
	_CurrentVersion = " ",
	_IsUpdating = false,
	_UpdatePhase = 0,
	_DotCount = 1,
	_UpdateCounter = 0,
	_CameraModelName = "",
	_FirmwareImageURL = "",
	_DefaultFirmwareImageURL = "",
	_FirmwareBinFile = "",
	_UpdateSucceeded = true,
	_OsVersionInfo = { version = "0.0.0" },
	_OsVersionCanDoUpdate = false,
}

function FirmwareControl:Initialize()
	local CallSucceeded, CallResult = pcall(C4:GetVersionInfo())

	if(CallSucceeded) then
		self._OsVersionInfo = CallResult
		self._OsVersionCanDoUpdate = self:OsVersionCanDoUpdate()
	end
	
	if(gCanUpdateFirmware) then
		if(self._OsVersionCanDoUpdate) then
			self._FirmwareUpdateMessageTimer = CreateTimer("Firmware Update", 4, "SECONDS", OnFirmwareUpdateMessageTimerExpired, true)
		end
	end
end


function FirmwareControl:OsVersionCanDoUpdate()
	-- There were a couple of bugs in the Url Put and Url Post code up until the 2.10.6 version.
	-- So any OS version up until 2.10.6 will fail. It was fixed then so 3.x.x or 2.10.x after that should
	-- be OK

	local RetVal = false
	local _, _, MajorRev, MinorRev, MiniRev = string.find(self._OsVersionInfo.version, "(%d+).(%d+).(%d+)")

	MajorRev = tonumber(MajorRev)
	MinorRev = tonumber(MinorRev)
	MiniRev = tonumber(MiniRev)

	if(MajorRev > 2) then
		RetVal = true
		
	elseif (MajorRev == 2) then
		if(MinorRev > 10) then
			RetVal = true
		
		elseif(MinorRev == 10) then
			RetVal = (MiniRev > 6)
		end
	end

	return RetVal
end


function FirmwareControl:SetVersion(NewVersion)
	self._CurrentVersion = NewVersion
	C4:UpdateProperty ('Firmware', self._CurrentVersion)
	LogInfo('Firmware Version: ' .. self._CurrentVersion)
end


function FirmwareControl:SetModelName(ModelName)
	self._CameraModelName = ModelName
	
	C4:UpdateProperty ('Camera Model Name', self._CameraModelName)
	LogInfo('--------------\nCamera Model: ))%s((', self._CameraModelName)
	
	if(#ModelName > 1) then
		self:UpdateModelFirmwareTarget()
		
		if(self._FirmwareImageURL == "") then
			self:SetDefaultFirmwareURL()
		end
	end
end

function FirmwareControl:GetModelName()
	return self._CameraModelName
end


function FirmwareControl:Update()
	LogTrace("FirmwareControl:Update")

	if(self._OsVersionCanDoUpdate) then
		self._IsUpdating = true
		Camera:ClearInfo()
		self._UpdatePhase = 1
		self:UpdateMessages()
		self._FirmwareUpdateMessageTimer:StartTimer()
		
		UrlGet(self._FirmwareImageURL, RetrievedFirmwareFromUrl)
	else
		LogWarn("Can't push a firmware update with this version of the OS: (%s)  Must do this manually.",
				self._OsVersionInfo.version)
		C4:UpdateProperty ('Firmware Update Progress', 
						   string.format("Auto update not supported. See documentation for manual update instructions"))
	end
end


function FirmwareControl:IsUpdating()
	return self._IsUpdating
end



function FirmwareControl:PushImageToCamera(FirmwareImage)
	self._UpdatePhase = 2

    -- Create multipart form-data Put data load...
    local boundary = "----------------PakedgeCamera-Upload-" .. tostring(math.random(1000000))

    gStartSection = '--' .. boundary .. '\r\n' .. 'Content-Disposition: form-data; name="file"; filename="' .. self._FirmwareBinFile .. '"\r\nContent-Type: application/octet-stream\r\n\r\n'
    gEndSection = '\r\n--' .. boundary .. '--' .. '\r\n'

    local PostData = gStartSection .. FirmwareImage .. gEndSection

    -- Create multipart/form-data and length headers...
    local PostHeaders = {}
    PostHeaders["Host"] = UrlGetAddress()
    PostHeaders["Connection"] = "keep-alive"
	PostHeaders["Content-Length"] = string.len(PostData)
    PostHeaders["Accept"] = "\*/\*"			-- */*
    PostHeaders["Content-Type"] = "multipart/form-data; boundary=" .. boundary
    PostHeaders["Origin"] = "http://" .. UrlGetAddress()

	UrlPost(FIRMWARE_UPGRADE_STRING, PostData, PostHeaders, FirmwarePushComplete)
end


function FirmwareControl:PushFinished()
	self._UpdatePhase = 3
	self._UpdateCounter = 30
	self._UpdateSucceeded = true
end


function FirmwareControl:RebootHappened()
	self._UpdatePhase = 4
	Camera:GetSettings()
end


function FirmwareControl:ResumedCommunication()
	self._FirmwareUpdateMessageTimer:KillTimer()
	self._IsUpdating = false
	ShowDriverInfo(self._UpdateSucceeded and "Firmware Update Complete" or "Firmware Update Failed")
	C4:UpdateProperty ('Firmware Update Progress', '-')

end

function FirmwareControl:UpdateAborted(AbortMsg)
	LogError("Firmware Update Aborted %s", AbortMsg)
	ShowDriverInfo("Firmware Update Failed")
	self._FirmwareUpdateMessageTimer:KillTimer()
	self._UpdateSucceeded = false
	C4:UpdateProperty ('Firmware Update Progress', "Firmware Update Aborted")
	Camera:GetSettings()
end

function FirmwareControl:SetFirmwareImageURL(TargetURL)
	self._FirmwareImageURL = TargetURL
	C4:UpdateProperty ('Firmware Image URL', self._FirmwareImageURL)
end


function FirmwareControl:SetDefaultFirmwareURL()
	self:SetFirmwareImageURL(self._DefaultFirmwareImageURL)
end


--=========================================================================

function FirmwareControl:UpdateMessages()

	local function MessageDots(BaseMsg)
		self._DotCount = self._DotCount + 1
		if(self._DotCount >= 6) then
			self._DotCount = 1
		end
	
		C4:UpdateProperty ('Firmware Update Progress', string.format("%s%s", BaseMsg, string.sub(".......", 1, self._DotCount)))
	end

	local InfoMessage = ""
	
	if(self._UpdatePhase == 1) then
		MessageDots("Retrieving Firmare Image")
		
	elseif(self._UpdatePhase == 2) then
		MessageDots("Pushing Firmware To Camera")
		
	elseif(self._UpdatePhase == 3) then
		MessageDots("Waiting For Camera Reboot")

		if(self._UpdateCounter > 0) then
			self._UpdateCounter = self._UpdateCounter - 1
		else
			self:RebootHappened()
		end
	
	elseif(self._UpdatePhase == 4) then
		MessageDots("Re-establishing Communication")
	
	else
		LogInfo("FirmwareControl:UpdateMessages  Invalid phase index: %d", tonumber(self._UpdatePhase))
	end

end


function OnFirmwareUpdateMessageTimerExpired()
	FirmwareControl:UpdateMessages()
end

--=========================================================================

function FirmwarePushComplete(ResponseCode)
	LogTrace("FirmwareUpdateComplete  ResponseCode is: %d", tonumber(ResponseCode))
	
	if(URL_ResponseIsValid(ResponseCode)) then
		FirmwareControl:PushFinished()
	else
		FirmwareControl:UpdateAborted(string.format("Unable to push image. Error: %s", GetURLResponseString(ResponseCode)))
	end
end


function RetrievedFirmwareFromUrl(ResponseCode, DataImage)
	LogTrace("RetrievedFirmwareFromUrl  ResponseCode is %d", tonumber(ResponseCode))
	LogDebug("Data Length is %d", #DataImage)
	
	if(URL_ResponseIsValid(ResponseCode)) then
		FirmwareControl:PushImageToCamera(DataImage)
	else
		FirmwareControl:UpdateAborted(string.format("Unable to retrieve image. Error: %s", GetURLResponseString(ResponseCode)))
	end
end



ld_pakedge_firmware = true		-- for development
