--[[=============================================================================
	File is: url_com.lua
	Copyright 2019 Control4 Corporation. All Rights Reserved.
===============================================================================]]

require "common.c4_utils"
require "lib.c4_object"
require "lib.c4_log"
require "modules.encryption"

-- Set template version for this file
if (TEMPLATE_VERSION ~= nil) then
	TEMPLATE_VERSION.url_com = "2019.04.25"
end

URL_DEFAULT_NETWORK_BINDING_ID = 6001
UrlCom = inheritsFrom(nil)


function UrlCom:construct()
	self._NameAndPassword = '-:-'
	self._IsInitialized = false

	-- Depracted, but keep for backward compatibilty-----------
	self._LastSendAction = ""
	self._LastSendUrl = ""
	self._LastSendData = ""
	self._LastSendHeaders = {}
	------------------------------------------------------------
end

function UrlCom:InitialSetup(UserName, Password, AuthRequired, AuthType, Address, HttpPort, RtspPort)

	if(PersistData._UrlPersist == nil) then
		PersistData._UrlPersist = {}
		
		PersistData._UrlPersist.ADDRESS					= Address
		PersistData._UrlPersist.HTTP_PORT               = HttpPort
		PersistData._UrlPersist.RTSP_PORT               = RtspPort
		PersistData._UrlPersist.AUTHENTICATION_REQUIRED = tostring(AuthRequired)
		PersistData._UrlPersist.AUTHENTICATION_TYPE     = AuthType
		PersistData._UrlPersist.USERNAME                = UserName
		PersistData._UrlPersist.PASSWORD                = Encryption:AsciiEncryptIt(Password)
		PersistData._UrlPersist.ENCRYPTED_PASSWORD		= true
		
		PersistData._UrlPersist.NETWORK_BINDING_ID		= URL_DEFAULT_NETWORK_BINDING_ID
	else
		-- check if this was persisted before we were encrypting the passwords.
		if(PersistData._UrlPersist.ENCRYPTED_PASSWORD == nil) then
			PersistData._UrlPersist.PASSWORD = Encryption:AsciiEncryptIt(PersistData._UrlPersist.PASSWORD)
			PersistData._UrlPersist.ENCRYPTED_PASSWORD		= true
		end
	end

	self._IsInitialized = true
	
	self:SetNameAndPassword()
end

-------------

function UrlCom:GetNetworkBindingID()
	return PersistData._UrlPersist.NETWORK_BINDING_ID
end


function UrlCom:SetNetworkBindingID(NewBindingId)
	PersistData._UrlPersist.NETWORK_BINDING_ID = NewBindingId
end

-------------

function UrlCom:GetAddress()
	return PersistData._UrlPersist.ADDRESS
end

function UrlCom:SetAddress(NewAddress)
	LogTrace("UrlCom:SetAddress: %s", NewAddress)

	if (NewAddress ~= PersistData._UrlPersist.ADDRESS) then
		PersistData._UrlPersist.ADDRESS = NewAddress
	end
end

-------------

function UrlCom:GetHttpPort()
	return PersistData._UrlPersist.HTTP_PORT
end

function UrlCom:SetHttpPort(NewPort)
	LogTrace("UrlCom:SetHttpPort: %s", tostring(NewPort))

	if (NewPort ~= PersistData._UrlPersist.HTTP_PORT) then
		PersistData._UrlPersist.HTTP_PORT = NewPort
	end
end

-------------

function UrlCom:GetRtspPort()
	return PersistData._UrlPersist.RTSP_PORT
end

function UrlCom:SetRtspPort(NewPort)
	LogTrace("UrlCom:SetRtspPort: %s", tostring(NewPort))

	if (NewPort ~= PersistData._UrlPersist.RTSP_PORT) then
		PersistData._UrlPersist.RTSP_PORT = NewPort
	end
end

-------------

function UrlCom:IsAuthenticationRequired()
	return toboolean(PersistData._UrlPersist.AUTHENTICATION_REQUIRED)
end

function UrlCom:SetAuthenticationRequired(IsRequired)
	PersistData._UrlPersist.AUTHENTICATION_REQUIRED = tostring(IsRequired)
end

-------------

function UrlCom:GetAuthenticationType()
	return PersistData._UrlPersist.AUTHENTICATION_TYPE
end

function UrlCom:SetAuthenticationType(NewType)
	PersistData._UrlPersist.AUTHENTICATION_TYPE = NewType

end

function UrlCom:AuthIsBasic()
	return (self:GetAuthenticationType() == "BASIC")
end

function UrlCom:AuthIsDigest()
	return (self:GetAuthenticationType() == "DIGEST")
end

-------------

function UrlCom:GetUserName()
	return PersistData._UrlPersist.USERNAME
end

function UrlCom:SetUserName(NewName)
	LogTrace("UrlCom:SetUserName: %s", tostring(NewName))

	if (NewName ~= PersistData._UrlPersist.USERNAME) then
		PersistData._UrlPersist.USERNAME = NewName
		self:SetNameAndPassword()

	end
end

-------------

function UrlCom:GetPassword()
	return Encryption:AsciiDecryptIt(PersistData._UrlPersist.PASSWORD)
end

function UrlCom:SetPassword(NewPassword)
	LogTrace("UrlCom:SetPassword")
	LogDev("Set password to: %s", tostring(NewPassword))
	
	if(self._IsInitialized) then
		if (NewPassword ~= self:GetPassword()) then
			PersistData._UrlPersist.PASSWORD = Encryption:AsciiEncryptIt(NewPassword)
			self:SetNameAndPassword()
		end
	end
end

function UrlCom:SetNameAndPassword()
	self._NameAndPassword = string.format("%s:%s", self:GetUserName(), self:GetPassword())
end

-------------
--[[
	Build HTTP request URL using global address and HTTP port variables and provided query string.
--]]
function UrlCom:BuildHTTP(query_string)
	return string.format("http://%s:%s/%s", self:GetAddress(), self:GetHttpPort(), query_string)
end


function UrlCom:AuthHeader(Inheader)
	local http_headers = Inheader or {}

	if self:IsAuthenticationRequired() then
		if (self:AuthIsBasic()) then		-- BASIC
			http_headers["Authorization"] = string.format("Basic %s", C4:Base64Encode(self._NameAndPassword))
			
		end
	end

	return http_headers
end


function UrlCom:AuthURL(InUrl)
	local RetUrl = InUrl
	
	if (self:IsAuthenticationRequired() and self:AuthIsDigest()) then
		RetUrl = string.format("%s%s@%s", string.sub(InUrl, 1, 7), self._NameAndPassword, string.sub(InUrl, 8))
	end
	
	return RetUrl
end




--[[
	Perform URL 'Get'
--]]
function UrlCom:Get(url, CallbackFunction)
	
	local MyCallback = CallbackFunction
	local DataImage = {}
	
	C4:url()
		:OnDone(function(transfer, responses, errCode, errMsg)
					
					if(errCode == 0) then
						LogInfo("GET Transfer succeeded")
						
					elseif(errCode == 1) then
						LogError("GET Transfer was aborted")
						
					else
						LogError("GET Transfer failed with error %d: %s  (%d responses completed)", tonumber(errCode), errMsg, #responses)
					end

					local ResponseCode = 0
					local Body = ""
					if(#responses > 0) then
						local LastResponse = responses[#responses]
						
						ResponseCode = LastResponse.code
--						Body = LastResponse.body
						Body = table.concat(DataImage)
				
						-- local ResponseHeaders = LastResponse.headers
						-- LogDebug(ResponseHeaders)
					end
					
					if (MyCallback ~= nil and type(MyCallback == "function")) then -- Need to parse response
						MyCallback(ResponseCode, Body)
					else
						LogInfo("GET C4:url():OnDone  No Callback defined")
					end

				end
			  )

		:SetOptions(	{
							["fail_on_error"] = false,
							["timeout"] = 120,
							["connect_timeout"] = 20
					    }
				   )
		
		
		:OnBody(function(transfer, response)
					-- LogTrace("C4:url():OnBody")
				end
			   )
		
		:OnBodyChunk(function(transfer, response, chunk)
						--LogTrace("Get:OnBodyChunk")
						table.insert(DataImage, chunk)
					 end
					)
		
		:Get(self:AuthURL(url), self:AuthHeader())
end



--[[
	Perform URL 'Put'
--]]
function UrlCom:Put(url, data, header, CallbackFunction)
	
	local MyCallback = CallbackFunction

	C4:url()
		:OnDone(function(transfer, responses, errCode, errMsg)
					
					if(errCode == 0) then
						LogInfo("PUT Transfer succeeded")

					elseif(errCode == 1) then
						LogError("PUT Transfer was aborted")

					else
						LogError("PUT Transfer failed with error %d: %s  (%d responses completed)", tonumber(errCode), errMsg, #responses)
					end

					local ResponseCode = 0
					if(#responses > 0) then
						local LastResponse = responses[#responses]
						
						ResponseCode = LastResponse.code
						local ResponseHeaders = LastResponse.headers

						LogDev(ResponseHeaders)
					end
						
					if (MyCallback ~= nil and type(MyCallback == "function")) then -- Need to parse response
						MyCallback(ResponseCode)
					else
						LogInfo("PUT C4:url():OnDone  No Callback defined")
					end
						
				end
			  )

		:SetOptions(	{
							["fail_on_error"] = false,
							["timeout"] = 120,
							["connect_timeout"] = 20
					    }
				   )
					   
		:Put(self:AuthURL(BuildHTTPString(url)), data, self:AuthHeader(header))
end



--[[
	Perform URL 'Post'
--]]
function UrlCom:Post(url, data, header, CallbackFunction)
	self:PostRaw(self:AuthURL(BuildHTTPString(url)), data, self:AuthHeader(header), CallbackFunction)
end


function UrlCom:PostRaw(url, data, header, CallbackFunction)
	
	local MyCallback = CallbackFunction

	C4:url()
		:OnDone(function(transfer, responses, errCode, errMsg)
					LogTrace("C4:url():Post():OnDone()")
					LogDev(transfer)
					LogDev(responses)
					
					if(errCode == 0) then
						LogInfo("POST Transfer succeeded")

					elseif(errCode == 1) then
						LogError("POST Transfer was aborted")

					else
						LogError("POST Transfer failed with error %d: %s  (%d responses completed)", tonumber(errCode), errMsg, #responses)
					end

					local ResponseCode = 0
					local ResponseBody = {}
					if(#responses > 0) then
						local LastResponse = responses[#responses]
						
						ResponseCode = LastResponse.code
						ResponseBody = LastResponse.body
						local ResponseHeaders = LastResponse.headers

						LogDev(ResponseHeaders)
					end
						
					if (MyCallback ~= nil and type(MyCallback == "function")) then -- Need to parse response
						MyCallback(ResponseCode, ResponseBody)
					else
						LogInfo("POST C4:url():OnDone  No Callback defined")
					end
						
				end
			  )

		:SetOptions(	{
							["fail_on_error"] = false,
							["timeout"] = 120,
							["connect_timeout"] = 20
					    }
				   )
					   
		:Post(url, data, header)
end


--[[============================================================================
	DCC 4/16/19
	The following calls have been depricated. Even though they mostly work, C4:urlGet,
	C4:urlPut, and C4:urlPost they have had some flakey issues.  
	
	The rewritten calls that use use C4:Url:Get :Put and :Post are more reliable.
	
	I'm afraid of breaking backward compatibility, so I don't dare just delete them yet.
--]]

--[[
	DEPRICATED!  Please use Get

	Perform URL 'Get" request using authentication if required.
	Returns ticket (number) that identifies response with 'ReceivedAsync' callback function.
--]]
function UrlCom:GetRequest(url)
	self._LastSendAction = "GET"
	self._LastSendUrl = url
	
	LogDev("UrlCom:GetRequest url: %s", self:AuthURL(url))
	LogDev(self:AuthHeader("GET", url))
	
	return C4:urlGet(self:AuthURL(url), self:AuthHeader(), false)
end


--[[
	DEPRICATED!  Please use Put
--]]
function UrlCom:PutRequest(url, data)
	self._LastSendAction = "PUT"
	self._LastSendUrl = url
	self._LastSendData = data
	
	return C4:urlPut(self:AuthURL(url), data, self:AuthHeader(), false)
end


--[[
	DEPRICATED!  Please use Post

	Perform URL 'Post" request using authentication if required.
	Returns ticket (number) that identifies response with 'ReceivedAsync' callback function.
--]]
function UrlCom:PostRequest(url, data, header)
	self._LastSendAction = "POST"
	self._LastSendUrl = url
	self._LastSendData = data
	self._LastSendHeaders = header
	
	return C4:urlPost(self:AuthURL(BuildHTTPString(url)), data, self:AuthHeader("POST", "/" .. url, header))

end



function UrlCom:ClearLastSend()
	UrlCom._LastSendAction = ""
	UrlCom._LastSendUrl = ""
	UrlCom._LastSendData = ""
	UrlCom._LastSendHeaders = {}
end

-------------------------------------------
--[[
	The ReceivedAsync function is called in response to 'URL.GetRequest'.
	The ticketId is the number returned from the request.
--]]
function ReceivedAsync(ticketId, strData, responseCode, tHeaders)
	strData = strData or ""
	responseCode = responseCode or 0
	tHeaders = tHeaders or {}

	--=======================================
	LogTrace("ReceivedAsync ticketID: %s  responseCode: %d (%s)  Length: %d", 
			 ticketId, responseCode, GetURLResponseString(responseCode), string.len(strData) )
	LogDev(tHeaders)
	LogDev(strData)
	--=======================================

	TheUrlCom:ClearLastSend()
	
	if (gTickets[ticketId] ~= nil and type(RESPONSES[gTickets[ticketId]]) == "function") then -- Need to parse response
		RESPONSES[gTickets[ticketId]](strData, responseCode)
		gTickets[ticketId] = nil
	end
end


ld_urlcom = true				-- for development
